require 'easy_type'

module Puppet_X
  module EnterpriseModules
    module Oracle
      class SqlplusDaemon < SqlplusCommand

        def initialize(options = {})
          super
          @catch_errors = Regexp.union(@catch_errors, /ORA-\d+:.*$/) if @failonsqlfail
          #
          # We don't need the sudo part of the returned command string, so
          # we remove it 
          #
          @daemon = EasyType::Daemon.run(identity)
          unless @daemon
            @command = command_string.split('"')[1]
            Puppet.info "Starting the Sqlplus daemon with os user #{@os_user} on sid #{@sid}"
            @daemon = EasyType::Daemon.new(identity, @command, @os_user)
            connect_to_oracle
          end
        end

        def execute_sql_command(command, output_file, timeout = nil)
          timeout ||= @timeout
          Puppet.debug "Executing daemonized sql-command #{command}"
          @daemon.execute_command template('ora_config/ora_config/daemon_execute.sql.erb', binding)
          @daemon.sync(timeout) do |line|
            if line =~ @catch_errors
              @daemon.sync(timeout) # Clear buffer till end string
              fail "Error in execution of SQL command: #{command}.\nFound error #{line}"
            end
          end
          File.read(output_file)
        end

        private

          def identity
            "ora-#{@sid}"
          end

          def connect_to_oracle
            Puppet.debug "Connecting to Oracle sid in daemon #{@sid} with os_user #{@os_user}"
            @daemon.execute_command template('ora_config/ora_config/daemon_connect.sql.erb', binding)
          end

      end
    end
  end
end
